@@ -279,6 +279,7 @@ impl<T, const N: usize> StackVec<T, N> {
     /// vec.truncate(4);
     /// assert_eq!(&vec[..], &[1, 2, 3]);
     /// ```
+    #[inline]
     pub fn truncate(&mut self, len: usize) {
         if len > self.len() {
             return;
@@ -312,6 +313,137 @@ impl<T, const N: usize> StackVec<T, N> {
     pub fn clear(&mut self) {
         self.truncate(0)
     }
+
+    /// Inserts an element at position `index` within the vector, shifting all
+    /// elements after it to the right.
+    ///
+    /// # Panics
+    ///
+    /// Panics if `index > len` or the vector is full.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use stack_buf::{StackVec, stack_vec};
+    ///
+    /// let mut vec = stack_vec![5#1, 2, 3];
+    /// vec.insert(1, 4);
+    /// assert_eq!(&vec[..], [1, 4, 2, 3]);
+    /// vec.insert(4, 5);
+    /// assert_eq!(&vec[..], [1, 4, 2, 3, 5]);
+    /// ```
+    #[inline]
+    pub fn insert(&mut self, index: usize, element: T) {
+        #[cold]
+        #[inline(never)]
+        fn assert_failed(index: usize, len: usize) -> ! {
+            panic!(
+                "insertion index (is {}) should be <= len (is {})",
+                index, len
+            );
+        }
+
+        let len = self.len();
+        if index > len {
+            assert_failed(index, len);
+        }
+        assert!(len < self.capacity());
+        unsafe {
+            let ptr = self.as_mut_ptr().add(index);
+            ptr::copy(ptr, ptr.offset(1), len - index);
+            ptr::write(ptr, element);
+            self.set_len(len + 1);
+        }
+    }
+
+    /// Removes and returns the element at position `index` within the vector,
+    /// shifting all elements after it to the left.
+    ///
+    /// # Panics
+    ///
+    /// Panics if `index` is out of bounds.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use stack_buf::StackVec;
+    ///
+    /// let mut vec = StackVec::from([1, 2, 3]);
+    /// assert_eq!(vec.remove(1), 2);
+    /// assert_eq!(&vec[..], [1, 3]);
+    /// ```
+    #[inline]
+    pub fn remove(&mut self, index: usize) -> T {
+        #[cold]
+        #[inline(never)]
+        fn assert_failed(index: usize, len: usize) -> ! {
+            panic!("removal index (is {}) should be < len (is {})", index, len);
+        }
+
+        let len = self.len();
+        if index >= len {
+            assert_failed(index, len);
+        }
+        unsafe {
+            let ret;
+            {
+                let ptr = self.as_mut_ptr().add(index);
+                ret = ptr::read(ptr);
+                ptr::copy(ptr.offset(1), ptr, len - index - 1);
+            }
+            self.set_len(len - 1);
+            ret
+        }
+    }
+
+    /// Removes an element from the vector and returns it.
+    ///
+    /// The removed element is replaced by the last element of the vector.
+    ///
+    /// This does not preserve ordering, but is O(1).
+    ///
+    /// # Panics
+    ///
+    /// Panics if `index` is out of bounds.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use stack_buf::StackVec;
+    ///
+    /// let mut v = StackVec::from(["foo", "bar", "baz", "qux"]);
+    ///
+    /// assert_eq!(v.swap_remove(1), "bar");
+    /// assert_eq!(&v[..], ["foo", "qux", "baz"]);
+    ///
+    /// assert_eq!(v.swap_remove(0), "foo");
+    /// assert_eq!(&v[..], ["baz", "qux"]);
+    /// ```
+    #[inline]
+    pub fn swap_remove(&mut self, index: usize) -> T {
+        #[cold]
+        #[inline(never)]
+        fn assert_failed(index: usize, len: usize) -> ! {
+            panic!(
+                "swap_remove index (is {}) should be < len (is {})",
+                index, len
+            );
+        }
+
+        let len = self.len();
+        if index >= len {
+            assert_failed(index, len);
+        }
+        unsafe {
+            // We replace self[index] with the last element. Note that if the
+            // bounds check above succeeds there must be a last element (which
+            // can be self[index] itself).
+            let last = ptr::read(self.as_ptr().add(len - 1));
+            let hole = self.as_mut_ptr().add(index);
+            self.set_len(len - 1);
+            ptr::replace(hole, last)
+        }
+    }
 }
 
 impl<T: Clone, const N: usize> StackVec<T, N> {
@@ -376,7 +508,7 @@ impl<T: Clone, const N: usize> StackVec<T, N> {
     ///
     /// ```
     /// use stack_buf::{StackVec, stack_vec};
-    /// let mut vec: StackVec<i32, 10> = stack_vec![1];
+    /// let mut vec = stack_vec![10#1];
     /// vec.extend_from_slice(&[2, 3, 4]);
     /// assert_eq!(vec.as_slice(), [1, 2, 3, 4]);
     /// ```
@@ -579,26 +711,43 @@ impl<const N: usize> std::io::Write for StackVec<u8, N> {
 ///
 /// ```
 /// use stack_buf::{StackVec, stack_vec};
+///
 /// let vec: StackVec<i32, 8> = stack_vec![];
 /// assert!(vec.is_empty());
+/// assert_eq!(vec.capacity(), 8);
+///
+/// let vec = stack_vec![i32; 16];
+/// assert!(vec.is_empty());
+/// assert_eq!(vec.capacity(), 16);
 /// ```
 ///
 /// - Creates a [`StackVec`] containing a given list of elements:
 ///
 /// ```
 /// use stack_buf::{StackVec, stack_vec};
-/// let vec: StackVec<_, 128> = stack_vec![1, 2, 3];
+///
+/// let vec = stack_vec![128#1, 2, 3];
+/// assert_eq!(vec.capacity(), 128);
 /// assert_eq!(vec[0], 1);
 /// assert_eq!(vec[1], 2);
 /// assert_eq!(vec[2], 3);
+///
+/// let vec = stack_vec![1, 2, 3];
+/// assert_eq!(vec.capacity(), 3);
+///
 /// ```
 ///
 /// - Creates a [`StackVec`] from a given element and size:
 ///
 /// ```
 /// use stack_buf::{StackVec, stack_vec};
-/// let v: StackVec<_, 0x8000> = stack_vec![1; 3];
+///
+/// let v = stack_vec![0x8000#1; 3];
 /// assert_eq!(v.as_slice(), [1, 1, 1]);
+/// assert_eq!(v.capacity(), 0x8000);
+///
+/// let v = stack_vec![1; 3];
+/// assert_eq!(v.capacity(), 3);
 /// ```
 ///
 /// Note that unlike array expressions this syntax supports all elements
@@ -613,11 +762,18 @@ impl<const N: usize> std::io::Write for StackVec<u8, N> {
 #[macro_export]
 macro_rules! stack_vec {
     () => ($crate::StackVec::new());
+    ($ty: ty; $cap: literal) => ($crate::StackVec::<$ty, $cap>::new());
     ($elem:expr; $n:expr) => ({
-        $crate::StackVec::from_elem($elem, $n)
+        $crate::StackVec::<_, $n>::from_elem($elem, $n)
+    });
+    ($cap:literal# $elem:expr; $n:expr) => ({
+        $crate::StackVec::<_, $cap>::from_elem($elem, $n)
     });
     ($($x:expr),+ $(,)?) => ({
-        let mut vec = $crate::StackVec::new();
+        $crate::StackVec::from([$($x),+])
+    });
+    ($cap:literal# $($x:expr),+ $(,)?) => ({
+        let mut vec = $crate::StackVec::<_, $cap>::new();
         $(vec.push($x);)+
         vec
     });
